(function() {

Type.registerNamespace('Leadtools.Annotations.Automation');

Leadtools.Annotations.Automation.MyAutomationControl = function Leadtools_Annotations_Automation_MyAutomationControl(backgroundCanvas) {
    if (backgroundCanvas == null) {
        throw new Error('backgroundCanvas can not be null');
    }
	
	var _this = this;
	
	//Create a canvas for the annotation container. We will render the annotations directly to this container
    this._annotationCanvas = document.createElement('canvas');
    this._annotationCanvas.width = backgroundCanvas.width;
    this._annotationCanvas.height = backgroundCanvas.height;
    this._annotationCanvas.id = backgroundCanvas.id + '_annotationCanvas';
    this._annotationCanvas.style.pixelLeft = 0;
    this._annotationCanvas.style.pixelTop = 0;
    this._annotationCanvas.style.marginTop = '0px';
    this._annotationCanvas.style.marginRight = 'auto';
    this._annotationCanvas.style.marginBottom = '0px';
    this._annotationCanvas.style.marginLeft = 'auto';
    this._annotationCanvas.style.position = 'absolute';
    this._annotationCanvas.style.zIndex = 100;

	this._annotationCanvas.addEventListener('gotfocus', function() {
        _this.handleGotFocus();
    }, false);
	
	this._annotationCanvas.addEventListener('lostfocus', function() {
        _this.handleLostFocus();
    }, false);
	
	//Add our annotation container to the parent of the background canvas
    backgroundCanvas.parentNode.appendChild(this._annotationCanvas);
	
	//We will use an InteractiveServive to detect all of the mouse/touch events
    this._service = Leadtools.Controls.InteractiveService.create(null, this._annotationCanvas, this._annotationCanvas);
    this._service.set_enableMouseWheel(false);
    this._service.set_dragStartsOnDown(true);

	this._service.add_dragStarted(function(sender, e) {
        _this._service_DragStarted(sender, e);
    });
	
	this._service.add_dragDelta(function(sender, e) {
        _this._service_DragDelta(sender, e);
    });
	
	this._service.add_dragCompleted(function(sender, e) {
        _this._service_DragCompleted(sender, e);
    });
	
	this._service.add_doubleTap(function(sender, e) {
        _this._service_DoubleTap(sender, e);
    });
	
    this._service.startListening();
    this._backgroundCanvas = backgroundCanvas;
    this._context = this._annotationCanvas.getContext('2d');
	window.addEventListener('resize', function() {
        _this.onSizeChanged();
    }, false);
}
Leadtools.Annotations.Automation.MyAutomationControl.prototype = {
    _hasFocus: false,
    _container: null,
    _engine: null,
    _backgroundCanvas: null,
    _annotationCanvas: null,
    _context: null,
    _service: null,
    
	//Call this method when the size or location of the background canvas changes so we can recalculate the size/location of the annotation canvas.
    onSizeChanged: function Leadtools_Annotations_Automation_MyAutomationControl$onSizeChanged() {
        var offset = Leadtools.LTHelper.getPosition(this._backgroundCanvas, null);
        this._annotationCanvas.style.left = offset.get_x().toString() + 'px';
        this._annotationCanvas.style.top = offset.get_y().toString() + 'px';
		
		//Notify the interactive service that the size changed
        this._service.onSizeChanged(Leadtools.LeadEventArgs.Empty);
    },
    
    handleLostFocus: function Leadtools_Annotations_Automation_MyAutomationControl$handleLostFocus(e) {
        this._hasFocus = false;
        if (this.__automationLostFocus != null) {
            this.__automationLostFocus(this._annotationCanvas, Leadtools.LeadEventArgs.Empty);
        }
    },
    
    handleGotFocus: function Leadtools_Annotations_Automation_MyAutomationControl$handleGotFocus(e) {
        this._hasFocus = true;
        if (this.__automationGotFocus != null) {
            this.__automationGotFocus(this._annotationCanvas, Leadtools.LeadEventArgs.Empty);
        }
    },
    
    _service_DoubleTap: function Leadtools_Annotations_Automation_MyAutomationControl$_service_DoubleTap(sender, e) {
        if (this.__automationDoubleClick != null) {
            this.__automationDoubleClick(this, Leadtools.Annotations.Core.AnnPointerEventArgs.create(Leadtools.Annotations.Core.AnnMouseButton.left, Leadtools.LeadPointD.get_empty()));
        }
    },
    
    _service_DragCompleted: function Leadtools_Annotations_Automation_MyAutomationControl$_service_DragCompleted(sender, e) {
        if (this.__automationPointerUp != null) {
            this.__automationPointerUp(this, Leadtools.Annotations.Core.AnnPointerEventArgs.create(Leadtools.Annotations.Core.AnnMouseButton.left, this.get_automationContainer().get_mapper().pointToContainerCoordinates(Leadtools.LeadPointD.create(e.get_position().get_x() - e.get_change().get_x(), e.get_position().get_y() - e.get_change().get_y()))));
        }
    },
    
    _service_DragDelta: function Leadtools_Annotations_Automation_MyAutomationControl$_service_DragDelta(sender, e) {
        if (this.__automationPointerMove != null) {
            this.__automationPointerMove(this, Leadtools.Annotations.Core.AnnPointerEventArgs.create(Leadtools.Annotations.Core.AnnMouseButton.left, this.get_automationContainer().get_mapper().pointToContainerCoordinates(Leadtools.LeadPointD.create(e.get_position().get_x() - e.get_change().get_x(), e.get_position().get_y() - e.get_change().get_y()))));
        }
    },
    
    _service_DragStarted: function Leadtools_Annotations_Automation_MyAutomationControl$_service_DragStarted(sender, e) {
        if (!this._hasFocus) {
            this.handleGotFocus(null);
        }
        if (this.__automationPointerDown != null) {
            this.__automationPointerDown(this, Leadtools.Annotations.Core.AnnPointerEventArgs.create(Leadtools.Annotations.Core.AnnMouseButton.left, this.get_automationContainer().get_mapper().pointToContainerCoordinates(Leadtools.LeadPointD.create(e.get_position().get_x() - e.get_change().get_x(), e.get_position().get_y() - e.get_change().get_y()))));
        }
    },
    
    get_automationContainer: function Leadtools_Annotations_Automation_MyAutomationControl$get_automationContainer() {
        return this._container;
    },
    
    get_automationDpiX: function Leadtools_Annotations_Automation_MyAutomationControl$get_automationDpiX() {
        return 96;
    },
    
    get_automationDpiY: function Leadtools_Annotations_Automation_MyAutomationControl$get_automationDpiY() {
        return 96;
    },
    
    get_automationEnabled: function Leadtools_Annotations_Automation_MyAutomationControl$get_automationEnabled() {
        return true;
    },
    
    get_automationSize: function Leadtools_Annotations_Automation_MyAutomationControl$get_automationSize() {
        return Leadtools.LeadSizeD.create(this._annotationCanvas.width, this._annotationCanvas.height);
    },
    
    add_automationSizeChanged: function Leadtools_Annotations_Automation_MyAutomationControl$add_automationSizeChanged(value) {
		Leadtools.LeadEventArgs.addToEvent(this, "automationSizeChanged", value);
    },
    remove_automationSizeChanged: function Leadtools_Annotations_Automation_MyAutomationControl$remove_automationSizeChanged(value) {
		Leadtools.LeadEventArgs.removeFromEvent(this, "automationSizeChanged", value);
    },
    
    __automationSizeChanged: null,
    
    get_automationTransform: function Leadtools_Annotations_Automation_MyAutomationControl$get_automationTransform() {
		//For this example, we will use the identity matrix. If you make any changes to the matrix of your background canvas, you should apply that same matrix here.
        return Leadtools.LeadMatrix.get_identity();
    },
    
    add_automationTransformChanged: function Leadtools_Annotations_Automation_MyAutomationControl$add_automationTransformChanged(value) {
		Leadtools.LeadEventArgs.addToEvent(this, "automationTransformChanged", value);
    },
    remove_automationTransformChanged: function Leadtools_Annotations_Automation_MyAutomationControl$remove_automationTransformChanged(value) {
        Leadtools.LeadEventArgs.removeFromEvent(this, "automationTransformChanged", value);
    },
    
    __automationTransformChanged: null,
    
    get_automationUseDpi: function Leadtools_Annotations_Automation_MyAutomationControl$get_automationUseDpi() {
        return false;
    },
    
    add_automationUseDpiChanged: function Leadtools_Annotations_Automation_MyAutomationControl$add_automationUseDpiChanged(value) {
        Leadtools.LeadEventArgs.addToEvent(this, "automationUseDpiChanged", value);
    },
    remove_automationUseDpiChanged: function Leadtools_Annotations_Automation_MyAutomationControl$remove_automationUseDpiChanged(value) {
        Leadtools.LeadEventArgs.removeFromEvent(this, "automationUseDpiChanged", value);
    },
    
    __automationUseDpiChanged: null,
    
    get_automationXResolution: function Leadtools_Annotations_Automation_MyAutomationControl$get_automationXResolution() {
        return 96;
    },
    
    get_automationYResolution: function Leadtools_Annotations_Automation_MyAutomationControl$get_automationYResolution() {
        return 96;
    },
    
    add_automationEnabledChanged: function Leadtools_Annotations_Automation_MyAutomationControl$add_automationEnabledChanged(value) {
        Leadtools.LeadEventArgs.addToEvent(this, "automationEnabledChanged", value);
    },
    remove_automationEnabledChanged: function Leadtools_Annotations_Automation_MyAutomationControl$remove_automationEnabledChanged(value) {
        Leadtools.LeadEventArgs.removeFromEvent(this, "automationEnabledChanged", value);
    },
    
    __automationEnabledChanged: null,
    
    add_automationGotFocus: function Leadtools_Annotations_Automation_MyAutomationControl$add_automationGotFocus(value) {
        Leadtools.LeadEventArgs.addToEvent(this, "automationGotFocus", value);
    },
    remove_automationGotFocus: function Leadtools_Annotations_Automation_MyAutomationControl$remove_automationGotFocus(value) {
        Leadtools.LeadEventArgs.removeFromEvent(this, "automationGotFocus", value);
    },
    
    __automationGotFocus: null,
    
    add_automationLostFocus: function Leadtools_Annotations_Automation_MyAutomationControl$add_automationLostFocus(value) {
        Leadtools.LeadEventArgs.addToEvent(this, "automationLostFocus", value);
    },
    remove_automationLostFocus: function Leadtools_Annotations_Automation_MyAutomationControl$remove_automationLostFocus(value) {
        Leadtools.LeadEventArgs.removeFromEvent(this, "automationLostFocus", value);
    },
    
    __automationLostFocus: null,
    
    add_automationDoubleClick: function Leadtools_Annotations_Automation_MyAutomationControl$add_automationDoubleClick(value) {
        Leadtools.LeadEventArgs.addToEvent(this, "automationDoubleClick", value);
    },
    remove_automationDoubleClick: function Leadtools_Annotations_Automation_MyAutomationControl$remove_automationDoubleClick(value) {
        Leadtools.LeadEventArgs.removeFromEvent(this, "automationDoubleClick", value);
    },
    
    __automationDoubleClick: null,
    
    add_automationPointerDown: function Leadtools_Annotations_Automation_MyAutomationControl$add_automationPointerDown(value) {
        Leadtools.LeadEventArgs.addToEvent(this, "automationPointerDown", value);
    },
    remove_automationPointerDown: function Leadtools_Annotations_Automation_MyAutomationControl$remove_automationPointerDown(value) {
        Leadtools.LeadEventArgs.removeFromEvent(this, "automationPointerDown", value);
    },
    
    __automationPointerDown: null,
    
    add_automationPointerMove: function Leadtools_Annotations_Automation_MyAutomationControl$add_automationPointerMove(value) {
        Leadtools.LeadEventArgs.addToEvent(this, "automationPointerMove", value);
    },
    remove_automationPointerMove: function Leadtools_Annotations_Automation_MyAutomationControl$remove_automationPointerMove(value) {
        Leadtools.LeadEventArgs.removeFromEvent(this, "automationPointerMove", value);
    },
    
    __automationPointerMove: null,
    
    add_automationPointerUp: function Leadtools_Annotations_Automation_MyAutomationControl$add_automationPointerUp(value) {
        Leadtools.LeadEventArgs.addToEvent(this, "automationPointerUp", value);
    },
    remove_automationPointerUp: function Leadtools_Annotations_Automation_MyAutomationControl$remove_automationPointerUp(value) {
        Leadtools.LeadEventArgs.removeFromEvent(this, "automationPointerUp", value);
    },
    
    __automationPointerUp: null,
    
    automationAttach: function Leadtools_Annotations_Automation_MyAutomationControl$automationAttach(container) {
        this._container = container;
        if (this._annotationCanvas != null) {
            if (this._context != null) {
				//Create a new rendererer that will render to our annotation canvas
                this._engine = new Leadtools.Annotations.Rendering.AnnHtml5RenderingEngine(this._container, this._context, false);
                if (this._engine != null) {
                    this._engine.render(Leadtools.LeadRectD.create(0, 0, this._annotationCanvas.width, this._annotationCanvas.height), true);
                }
            }
        }
    },
    
    automationDetach: function Leadtools_Annotations_Automation_MyAutomationControl$automationDetach() {
        this._container = null;
    },
    
    automationInvalidate: function Leadtools_Annotations_Automation_MyAutomationControl$automationInvalidate(rc) {
        if (this._engine != null) {
            this._engine.render(Leadtools.LeadRectD.create(0, 0, this._annotationCanvas.width, this._annotationCanvas.height), true);
        }
    },
    
    get_automationOffset: function Leadtools_Annotations_Automation_MyAutomationControl$get_automationOffset() {
        return Leadtools.LeadPointD.create(0, 0);
    },
    
    get_renderingEngine: function Leadtools_Annotations_Automation_MyAutomationControl$get_renderingEngine() {
        return this._engine;
    },
    
    getImageData: function Leadtools_Annotations_Automation_MyAutomationControl$getImageData(rc) {
        if (rc.get_isEmpty()) {
            return null;
        }
        var context = this._backgroundCanvas.getContext('2d');
        return context.getImageData(parseInt(rc.get_left()), parseInt(rc.get_top()), parseInt(rc.get_width()), parseInt(rc.get_height()));
    },
    
    putImageData: function Leadtools_Annotations_Automation_MyAutomationControl$putImageData(data, position) {
        if (position.get_isEmpty() || data == null) {
            return;
        }
        var context = this._backgroundCanvas.getContext('2d');
        var pixelData = data.data;
        context.putImageData(data, parseInt(position.get_x()), parseInt(position.get_y()));
    }
}


Leadtools.Annotations.Automation.MyAutomationControl.registerClass('Leadtools.Annotations.Automation.MyAutomationControl', null, Leadtools.Annotations.Core.IAnnAutomationControl);
})();
